#include "main.h"
#include <QTextStream>
#include <QString>
#include <QStringList>
#include "qstringtools.h"
#include <QCryptographicHash>

int main(int argc,char** argv) {
    QTextStream out(stdout);
    bool quiet = false;
    int lenght = 8;
    QString startPassPhrase;

    QCryptographicHash::Algorithm mode = QCryptographicHash::Md5;
    //definition du pattern par defaut.
    if (pattern.isNull()) pattern = QString(PATTERN_ALPHA_NUMERIC_LOWER);


    QStringList tokens = parseCom(argc,argv);
    const int m = tokens.count();
    int i = 0;
    foreach (QString param,tokens) {
        i++;
        if (param == "-p") {
            if (i > tokens.count())  pattern = QString(PATTERN_PASSWORD);
            else  pattern = QStringTools::removeDoublesChar(tokens.at(i));
            out << "New pattern: " << pattern << endl;
        }
        else if (param == "-s") pattern = QStringTools::removeDoublesChar(pattern + " ");
        else if (param == "-a") pattern = QString(PATTERN_PASSWORD);
        else if (param == "-l") {
            //changement de la longueur du pass
            if (m <= i) return 1;
            lenght = tokens.at(i).toInt();
            if (!quiet) out << "new lenght: " << lenght << endl;
        }
        else if (param == "-m") {
            //fonction de bruteforce du hash donné en argument

            if (tokens.count() <= i) return 0;

            const QString currentHash = tokens.at(i);
            QString currentPass = pattern.at(0);

            if (!QStringTools::matchPattern(currentHash,PATTERN_HEXADECIMAL)) {
                out << "Error: invalid hash: not hexadecial" << endl;
                return 1;
            }
            else if (currentHash.length() == 40) mode = QCryptographicHash::Sha1;
            else if (currentHash.length() == 32) mode = QCryptographicHash::Md5;
            else {
                out << "Lenght: " << currentHash.length() << endl;
                out << "Error: the hash size is invalid: " << currentHash << endl;
                return 1;
            }

            QCryptographicHash hash(mode);

            int c = 0;
            while (true) {
                hash.addData(currentPass.toLocal8Bit());
                QByteArray result = hash.result();
                if (!quiet) out << ++c << ": " << result.toHex() << " -> " << currentPass << endl;
                if (result.toHex() == currentHash) {
                    out << "Password found: " << currentPass << endl;
                    out << "Hash: " << currentHash << endl;
                    break;
                }
                hash.reset();
                QStringTools::nextString(&currentPass,&pattern);
            }
            return 0;
        }
        else if (param == "-q") {
            out << "Using quiet mode" << endl;
            quiet = true;
        }
        else if (param == "-x") {
            //pattern = PATTERN_NUMERIC;
            out << "Pattern: " << pattern << endl;
            out << QStringTools::getStringAt(11,pattern) << endl;
        }
        else if (param == "-e") {
            //estimate the number of possibilities
            out << "Pattern: " << pattern << " (" << pattern.length() << ")" << endl;
            out << "For lenght: " << lenght << endl;
            out << "Password possibilities: " << QString().number(QStringTools::getPasswordPossibility(pattern,lenght)) << endl;
            return 0;
        }
        else if ((param == "-hash") || (param == "--hash")) {
            //cette fonction sers à avoir le hash d'un mot de passe donné en argument.

            if (tokens.count() <= i) return 0;
            QList<QCryptographicHash::Algorithm> modes;
            modes << QCryptographicHash::Md5;
            modes << QCryptographicHash::Sha1;
            foreach (QCryptographicHash::Algorithm currentMode,modes) {
                QCryptographicHash hash(currentMode);
                hash.addData(tokens.at(i).toLocal8Bit());
                out << currentMode << ": " << hash.result().toHex() << endl;
            }
            out << "Done." << endl;
        }

        else if (param == "-h") {
            out << "Valid options are:" << endl;
            out << "-h : show this help" << endl;
            out << "-s : put a 'space' in the pattern" << endl;
            out << "-a : set the pattern for 'passwords' (all commons chars)" << endl;
            out << "-m <hash> : try to break the password of the given hash (md5 or sha1)" << endl;
            out << "-g : give raw passwords only (not hash) (to make bruteforce dictionary)" << endl;
            out << "-l <lenght> : set the start lenght" << endl;
            out << "-e : estimate the number of possibilities" << endl;
            out << "--bbox : genererate old bbox keys" << endl;
        }
        else if (param == "-g") {
            //la longueur minimale du pass sera de 8 chars: les mots de passe wifi font entre 8 et 63
            QString keyB = QStringTools::strstr(pattern.at(0),lenght);

            while (keyB.length() < 64) {
                QStringTools::nextString(&keyB,&pattern);
                out << keyB << endl;
            }
        }
        else if (param == "-r") {
            //genere des mdp aléatoires avec le pattern;
            //note: une live box wpa pourait etre senssible à quelque chose comme: ./passgen -l 26 -p 012456789ABCDEF -r > /dev/shm/dico.txt

            while (true) {
                out << QStringTools::randString(pattern,lenght) << endl;
            }
        }
        else if (param == "-s") {
            if (m <= i) return 1;
            startPassPhrase = tokens.at(i);
            lenght = startPassPhrase.length();
            out << "Start passphrase: " << startPassPhrase << endl;
            out << "Lenght: " << lenght << endl;
        }
        else if (param == "--bbox") {
            //cette fonction génére tous les mdp par defaut possibles pour les bbox.
            //soit 16 982 783: mdp

            pattern = PATTERN_ALPHA_NUMERIC_LOWER;
            pattern = pattern.toUpper();
            bboxPassGen(out,pattern);
            return 0;
        }
    }
    return 0;
}

void bboxPassGen(QTextStream &out,const QString &pattern,const bool showEssid) {
    //cette fonction génére tous les mdp par defaut possibles pour les bbox. (ancienes générations :( )
    //soit 16 982 783: mdp
    QString yy,ww,xxx,all,sha1,essid,wpaKey;

    for (int year = 6;year < 13;year++) {
        yy = QString().number(year).rightJustified(2,'0');
        for (int week = 52;week;week--) {
            ww = QString().number(week).rightJustified(2,'0');
            xxx = "000";
            do {
                QStringTools::nextString(&xxx,&pattern);
                all = "CP" + yy + ww + QByteArray(xxx.toLocal8Bit()).toHex();

                QCryptographicHash hash(QCryptographicHash::Sha1);
                hash.addData(all.toLocal8Bit());

                sha1 = hash.result().toHex();

                if (showEssid) {
                    essid = "SpeedTouch" + sha1.mid(sha1.length() -6).toUpper();
                    out << "essid: " << essid << " password: ";
                }
                wpaKey = sha1.mid(0,10).toUpper();

                out << wpaKey << endl;
            }
            while (xxx.length() == 3);
        }
    }
}

QStringList parseCom(const int &argc,char** argv) {
    QString rawLine;
    for (int p = 1;p < argc;p++) rawLine += QString(" ") + argv[p];
    rawLine.remove(0,1);

    return rawLine.split(' ');
}
